@@ -1,5 +1,5 @@
 /* Tcl/Tk command definitions for Insight.
-   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2001, 2002, 2003, 2004
+   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2001, 2002, 2003, 2004, 2007
    Free Software Foundation, Inc.
 
    Written by Stu Grossman <grossman@cygnus.com> of Cygnus Support.
@@ -39,6 +39,7 @@
 #include "filenames.h"
 #include "disasm.h"
 #include "value.h"
+#include "varobj.h"
 #include "exceptions.h"
 
 /* tcl header files includes varargs.h unless HAS_STDARG is defined,
@@ -1894,7 +1895,7 @@ gdbtk_load_asm (ClientData clientData, CORE_ADDR pc,
 
   result_ptr->obj_ptr = client_data->result_obj[2];
   /* FIXME: cagney/2003-09-08: This should use gdb_disassembly.  */
-  insn = gdb_print_insn (pc, gdb_stdout);
+  insn = gdb_print_insn (pc, gdb_stdout, NULL);
   gdb_flush (gdb_stdout);
 
   client_data->widget_line_no++;
@@ -2129,23 +2130,29 @@ gdb_loc (ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST obj
           pc = entry_point_address ();
           sal = find_pc_line (pc, 0);
         }  
-      else if (deprecated_selected_frame
-	       && (get_frame_pc (deprecated_selected_frame) != read_pc ()))
-        {
-          /* Note - this next line is not correct on all architectures.
-	     For a graphical debugger we really want to highlight the 
-	     assembly line that called the next function on the stack.
-	     Many architectures have the next instruction saved as the
-	     pc on the stack, so what happens is the next instruction 
-	     is highlighted. FIXME */
-	  pc = get_frame_pc (deprecated_selected_frame);
-	  find_frame_sal (deprecated_selected_frame, &sal);
-	}
       else
-        {
-          pc = read_pc ();
-          sal = find_pc_line (pc, 0);
-        }
+	{
+	  struct frame_info *frame;
+
+	  frame = get_selected_frame (NULL);
+
+	  if (get_frame_pc (frame) != read_pc ())
+	    {
+	      /* Note - this next line is not correct on all architectures.
+		 For a graphical debugger we really want to highlight the 
+		 assembly line that called the next function on the stack.
+		 Many architectures have the next instruction saved as the
+		 pc on the stack, so what happens is the next instruction 
+		 is highlighted. FIXME */
+	      pc = get_frame_pc (frame);
+	      find_frame_sal (frame, &sal);
+	    }
+	  else
+	    {
+	      pc = read_pc ();
+	      sal = find_pc_line (pc, 0);
+	    }
+	}
     }
   else if (objc == 2)
     {
@@ -2252,7 +2259,7 @@ hex2bin (const char *hex, char *bin, int count)
   int incr = 2;
 
 
-  if (TARGET_BYTE_ORDER == BFD_ENDIAN_LITTLE)
+  if (gdbarch_byte_order (current_gdbarch) == BFD_ENDIAN_LITTLE)
     {
       /* need to read string in reverse */
       hex += count - 2;
@@ -2429,15 +2436,12 @@ gdb_update_mem (ClientData clientData, Tcl_Interp *interp,
   memset (mbuf, 0, nbytes + 32);
   mptr = cptr = mbuf;
 
-  rnum = 0;
-  while (rnum < nbytes)
+  rnum = target_read (&current_target, TARGET_OBJECT_MEMORY, NULL,
+		      mbuf, addr, nbytes);
+  if (rnum <= 0)
     {
-      int error;
-      int num = target_read_memory_partial (addr + rnum, mbuf + rnum,
-					    nbytes - rnum, &error);
-      if (num <= 0)
-	break;
-      rnum += num;
+      gdbtk_set_result (interp, "Unable to read memory.");
+      return TCL_ERROR;
     }
 
   if (objc == 8)
