@@ -160,9 +160,10 @@ proc pref_read {} {
 #  PROC:  pref_save - save preferences to a file and delete window
 # ------------------------------------------------------------------
 proc pref_save {{win {}}} {
-  global prefs_init_filename GDBStartup
+  global prefs_init_filename GDBStartup env
 
-  if {!$GDBStartup(inhibit_prefs)} {
+  if {!$GDBStartup(inhibit_prefs)
+      || [info exists env(INSIGHT_FORCE_READ_PREFERENCES)]} {
     debug "pref_save $prefs_init_filename"
 
     if {[catch {open $prefs_init_filename w} fd]} {
@@ -440,7 +441,7 @@ proc pref_set_defaults {} {
   # IPC prefs
   # set prefs based on GDB version?
   #set vers [lindex [split [lindex [split [gdb_cmd "show version"]] end-1 ] \"] 1]
-  pref define gdb/ipc/enabled	1
+  pref define gdb/ipc/enabled		0
   pref define gdb/ipc/port		9909
   pref define gdb/ipc/stop_button	1
   pref define gdb/ipc/step_button	1
